from typing import Any, Dict, Optional

import pytest

from knot_resolver.datamodel.management_schema import ManagementSchema
from knot_resolver.utils.modeling.exceptions import DataValidationError


@pytest.mark.parametrize("val", [{"interface": "::1@53"}, {"unix-socket": "/tmp/socket"}])
def test_management_valid(val: Dict[str, Any]):
    o = ManagementSchema(val)
    if o.interface:
        assert str(o.interface) == val["interface"]
    if o.unix_socket:
        assert str(o.unix_socket) == val["unix-socket"]


@pytest.mark.parametrize("val", [None, {"interface": "::1@53", "unix-socket": "/tmp/socket"}])
def test_management_invalid(val: Optional[Dict[str, Any]]):
    with pytest.raises(DataValidationError):
        ManagementSchema(val)



import pytest
from pytest import raises

from knot_resolver.datamodel.forward_schema import ForwardSchema
from knot_resolver.utils.modeling.exceptions import DataValidationError


@pytest.mark.parametrize("port,auth", [(5335, False), (53, True)])
def test_forward_valid(port: int, auth: bool):
    assert ForwardSchema(
        {"subtree": ".", "options": {"authoritative": auth, "dnssec": True}, "servers": [f"127.0.0.1", "::1"]}
    )
    assert ForwardSchema(
        {"subtree": ".", "options": {"authoritative": auth, "dnssec": False}, "servers": [f"127.0.0.1@{port}", "::1"]}
    )

    assert ForwardSchema(
        {
            "subtree": ".",
            "options": {"authoritative": auth, "dnssec": False},
            "servers": [{"address": [f"127.0.0.1@{port}", "::1"]}],
        }
    )

    assert ForwardSchema(
        {
            "subtree": ".",
            "options": {"authoritative": auth, "dnssec": False},
            "servers": [{"address": [f"127.0.0.1", "::1"]}],
        }
    )


@pytest.mark.parametrize(
    "port,auth,tls",
    [(5335, True, False), (53, True, True)],
)
def test_forward_invalid(port: int, auth: bool, tls: bool):
    if not tls:
        with raises(DataValidationError):
            ForwardSchema(
                {
                    "subtree": ".",
                    "options": {"authoritative": auth, "dnssec": False},
                    "servers": [f"127.0.0.1@{port}", "::1"],
                }
            )

    with raises(DataValidationError):
        ForwardSchema(
            {
                "subtree": ".",
                "options": {"authoritative": auth, "dnssec": False},
                "servers": [{"address": [f"127.0.0.1{port}", f"::1{port}"], "transport": "tls" if tls else None}],
            }
        )



from typing import Any, Dict, List, Literal, Optional, Tuple, Type, Union

import pytest
from pytest import raises

from knot_resolver.utils.modeling import ConfigSchema, parse_json, parse_yaml
from knot_resolver.utils.modeling.exceptions import DataDescriptionError, DataValidationError


class _TestBool(ConfigSchema):
    v: bool


class _TestInt(ConfigSchema):
    v: int


class _TestStr(ConfigSchema):
    v: str


class _TestLiteral(ConfigSchema):
    v: Literal[Literal["lit1"], Literal["lit2"]]


@pytest.mark.parametrize("val", ["lit1", "lit2"])
def test_parsing_literal_valid(val: str):
    assert _TestLiteral(parse_yaml(f"v: {val}")).v == val


@pytest.mark.parametrize("val", ["invalid", "false", 1, "null"])
def test_parsing_literal_invalid(val: str):
    with raises(DataValidationError):
        _TestLiteral(parse_yaml(f"v: {val}"))


@pytest.mark.parametrize("val,exp", [("false", False), ("true", True), ("False", False), ("True", True)])
def test_parsing_bool_valid(val: str, exp: bool):
    assert _TestBool(parse_yaml(f"v: {val}")).v == exp


@pytest.mark.parametrize("val", ["0", "1", "5", "'true'", "'false'", "5.5"])  # int, str, float
def test_parsing_bool_invalid(val: str):
    with raises(DataValidationError):
        _TestBool(parse_yaml(f"v: {val}"))


@pytest.mark.parametrize("val,exp", [("0", 0), ("5335", 5335), ("-5001", -5001)])
def test_parsing_int_valid(val: str, exp: int):
    assert _TestInt(parse_yaml(f"v: {val}")).v == exp


@pytest.mark.parametrize("val", ["false", "'5'", "5.5"])  # bool, str, float
def test_parsing_int_invalid(val: str):
    with raises(DataValidationError):
        _TestInt(parse_yaml(f"v: {val}"))


# int and float are allowed inputs for string
@pytest.mark.parametrize("val,exp", [("test", "test"), (65, "65"), (5.5, "5.5")])
def test_parsing_str_valid(val: Any, exp: str):
    assert _TestStr(parse_yaml(f"v: {val}")).v == exp


def test_parsing_str_invalid():
    with raises(DataValidationError):
        _TestStr(parse_yaml("v: false"))  # bool


def test_parsing_list_empty():
    class ListSchema(ConfigSchema):
        empty: List[Any]

    with raises(DataValidationError):
        ListSchema(parse_yaml("empty: []"))


@pytest.mark.parametrize("typ,val", [(_TestInt, 5), (_TestBool, False), (_TestStr, "test")])
def test_parsing_nested(typ: Type[ConfigSchema], val: Any):
    class UpperSchema(ConfigSchema):
        l: typ

    yaml = f"""
l:
  v: {val}
"""

    o = UpperSchema(parse_yaml(yaml))
    assert o.l.v == val


def test_parsing_simple_compound_types():
    class TestSchema(ConfigSchema):
        l: List[int]
        d: Dict[str, str]
        t: Tuple[str, int]
        o: Optional[int]

    yaml = """
l:
  - 1
  - 2
  - 3
  - 4
  - 5
d:
  something: else
  w: all
t:
  - test
  - 5
"""

    o = TestSchema(parse_yaml(yaml))
    assert o.l == [1, 2, 3, 4, 5]
    assert o.d == {"something": "else", "w": "all"}
    assert o.t == ("test", 5)
    assert o.o is None


def test_parsing_nested_compound_types():
    class TestSchema(ConfigSchema):
        i: int
        o: Optional[Dict[str, str]]

    yaml1 = "i: 5"
    yaml2 = f"""
{yaml1}
o:
  key1: str1
  key2: str2
    """

    o = TestSchema(parse_yaml(yaml1))
    assert o.i == 5
    assert o.o is None

    o = TestSchema(parse_yaml(yaml2))
    assert o.i == 5
    assert o.o == {"key1": "str1", "key2": "str2"}


def test_dash_conversion():
    class TestSchema(ConfigSchema):
        awesome_field: Dict[str, str]

    yaml = """
awesome-field:
  awesome-key: awesome-value
"""

    o = TestSchema(parse_yaml(yaml))
    assert o.awesome_field["awesome-key"] == "awesome-value"


def test_eq():
    class B(ConfigSchema):
        a: _TestInt
        field: str

    b1 = B({"a": {"v": 6}, "field": "val"})
    b2 = B({"a": {"v": 6}, "field": "val"})
    b_diff = B({"a": {"v": 7}, "field": "val"})

    assert b1 == b2
    assert b2 != b_diff
    assert b1 != b_diff
    assert b_diff == b_diff


def test_docstring_parsing_valid():
    class NormalDescription(ConfigSchema):
        """
        Does nothing special
        Really
        """

    desc = NormalDescription.json_schema()
    assert desc["description"] == "Does nothing special\nReally"

    class FieldsDescription(ConfigSchema):
        """
        This is an awesome test class
        ---
        field: This field does nothing interesting
        value: Neither does this
        """

        field: str
        value: int

    schema = FieldsDescription.json_schema()
    assert schema["description"] == "This is an awesome test class"
    assert schema["properties"]["field"]["description"] == "This field does nothing interesting"
    assert schema["properties"]["value"]["description"] == "Neither does this"

    class NoDescription(ConfigSchema):
        nothing: str

    _ = NoDescription.json_schema()


def test_docstring_parsing_invalid():
    class AdditionalItem(ConfigSchema):
        """
        This class is wrong
        ---
        field: nope
        nothing: really nothing
        """

        nothing: str

    with raises(DataDescriptionError):
        _ = AdditionalItem.json_schema()

    class WrongDescription(ConfigSchema):
        """
        This class is wrong
        ---
        other: description
        """

        nothing: str

    with raises(DataDescriptionError):
        _ = WrongDescription.json_schema()



from typing import Any

import pytest
from pytest import raises

from knot_resolver.datamodel.local_data_schema import RuleSchema
from knot_resolver.utils.modeling.exceptions import DataValidationError


@pytest.mark.parametrize(
    "val",
    [
        {"name": ["sub2.example.org"], "subtree": "empty", "tags": ["t01"]},
        {"name": ["sub3.example.org", "sub5.example.net."], "subtree": "nxdomain", "ttl": "1h"},
        {"name": ["sub4.example.org"], "subtree": "redirect"},
        {"name": ["sub5.example.org"], "address": ["127.0.0.1"]},
        {"name": ["sub6.example.org"], "subtree": "redirect", "address": ["127.0.0.1"]},
        {"file": "/etc/hosts", "ttl": "20m", "nodata": True},
        {"records": "", "ttl": "20m", "nodata": True},
    ],
)
def test_subtree_valid(val: Any):
    RuleSchema(val)


@pytest.mark.parametrize(
    "val",
    [
        {"subtree": "empty"},
        {"name": ["sub2.example.org"], "file": "/etc/hosts"},
        {"name": ["sub4.example.org"], "address": ["127.0.0.1"], "subtree": "nxdomain"},
        {"name": ["sub4.example.org"], "subtree": "redirect", "file": "/etc/hosts"},
    ],
)
def test_subtree_invalid(val: Any):
    with raises(DataValidationError):
        RuleSchema(val)



from typing import Any, List, Optional, Union

import pytest
from pytest import raises

from knot_resolver.datamodel.types import ListOrItem
from knot_resolver.utils.modeling import BaseSchema
from knot_resolver.utils.modeling.exceptions import DataValidationError
from knot_resolver.utils.modeling.types import get_generic_type_wrapper_argument


@pytest.mark.parametrize("val", [str, int])
def test_list_or_item_inner_type(val: Any):
    assert get_generic_type_wrapper_argument(ListOrItem[val]) == Union[List[val], val]


@pytest.mark.parametrize(
    "typ,val",
    [
        (int, [1, 65_535, 5335, 5000]),
        (int, 65_535),
        (str, ["string1", "string2"]),
        (str, "string1"),
    ],
)
def test_list_or_item_valid(typ: Any, val: Any):
    class ListOrItemSchema(BaseSchema):
        test: ListOrItem[typ]

    o = ListOrItemSchema({"test": val})
    assert o.test.serialize() == val
    assert o.test.to_std() == val if isinstance(val, list) else [val]

    i = 0
    for item in o.test:
        assert item == val[i] if isinstance(val, list) else val
        i += 1


@pytest.mark.parametrize(
    "typ,val",
    [
        (str, [True, False, True, False]),
        (str, False),
        (bool, [1, 65_535, 5335, 5000]),
        (bool, 65_535),
        (int, "string1"),
        (int, ["string1", "string2"]),
    ],
)
def test_list_or_item_invalid(typ: Any, val: Any):
    class ListOrItemSchema(BaseSchema):
        test: ListOrItem[typ]

    with raises(DataValidationError):
        ListOrItemSchema({"test": val})


def test_list_or_item_empty():
    with raises(ValueError):
        ListOrItem([])



from typing import Any, Dict, Optional

import pytest
from pytest import raises

from knot_resolver.constants import WATCHDOG_LIB
from knot_resolver.datamodel.network_schema import ListenSchema, NetworkSchema, TLSSchema
from knot_resolver.datamodel.types import InterfaceOptionalPort, PortNumber
from knot_resolver.utils.modeling.exceptions import DataValidationError


def test_listen_defaults():
    o = NetworkSchema()

    assert len(o.listen) == 2
    # {"ip-address": "127.0.0.1"}
    assert o.listen[0].interface.to_std() == [InterfaceOptionalPort("127.0.0.1")]
    assert o.listen[0].port == PortNumber(53)
    assert o.listen[0].kind == "dns"
    assert o.listen[0].freebind == False
    # {"ip-address": "::1", "freebind": True}
    assert o.listen[1].interface.to_std() == [InterfaceOptionalPort("::1")]
    assert o.listen[1].port == PortNumber(53)
    assert o.listen[1].kind == "dns"
    assert o.listen[1].freebind == True


@pytest.mark.parametrize(
    "listen,port",
    [
        ({"unix-socket": ["/tmp/kresd-socket"]}, None),
        ({"interface": ["::1"]}, 53),
        ({"interface": ["::1"], "kind": "dot"}, 853),
        ({"interface": ["::1"], "kind": "doh-legacy"}, 443),
        ({"interface": ["::1"], "kind": "doh2"}, 443),
        ({"interface": ["::1"], "kind": "doq"}, 853),
    ],
)
def test_listen_port_defaults(listen: Dict[str, Any], port: Optional[int]):
    assert ListenSchema(listen).port == (PortNumber(port) if port else None)


@pytest.mark.parametrize(
    "listen",
    [
        {"unix-socket": "/tmp/kresd-socket"},
        {"unix-socket": ["/tmp/kresd-socket", "/tmp/kresd-socket2"]},
        {"interface": "::1"},
        {"interface": "::1@5335"},
        {"interface": "::1", "port": 5335},
        {"interface": ["127.0.0.1", "::1"]},
        {"interface": ["127.0.0.1@5335", "::1@5335"]},
        {"interface": ["127.0.0.1", "::1"], "port": 5335},
        {"interface": "lo"},
        {"interface": "lo@5335"},
        {"interface": "lo", "port": 5335},
        {"interface": ["lo", "eth0"]},
        {"interface": ["lo@5335", "eth0@5335"]},
        {"interface": ["lo", "eth0"], "port": 5335},
    ],
)
def test_listen_valid(listen: Dict[str, Any]):
    assert ListenSchema(listen)


@pytest.mark.parametrize(
    "listen",
    [
        {"unix-socket": "/tmp/kresd-socket", "port": "53"},
        {"interface": "::1", "unix-socket": "/tmp/kresd-socket"},
        {"interface": "::1@5335", "port": 5335},
        {"interface": ["127.0.0.1", "::1@5335"]},
        {"interface": ["127.0.0.1@5335", "::1@5335"], "port": 5335},
        {"interface": "lo@5335", "port": 5335},
        {"interface": ["lo", "eth0@5335"]},
        {"interface": ["lo@5335", "eth0@5335"], "port": 5335},
    ],
)
def test_listen_invalid(listen: Dict[str, Any]):
    with raises(DataValidationError):
        ListenSchema(listen)


@pytest.mark.parametrize(
    "tls",
    [
        {"watchdog": "auto"},
        {"watchdog": True},
        {"watchdog": False},
    ],
)
def test_tls_watchdog(tls: Dict[str, Any]):
    expected: bool = WATCHDOG_LIB if tls["watchdog"] == "auto" else tls["watchdog"]
    assert TLSSchema(tls).watchdog == expected



from typing import Any

import pytest
from jinja2 import Template

from knot_resolver.datamodel.types import EscapedStr
from knot_resolver.utils.modeling import ConfigSchema

str_template = Template("'{{ string }}'")


str_multiline_template = Template(
    """[[
{{ string.multiline() }}
]]"""
)


@pytest.mark.parametrize(
    "val,exp",
    [
        ("\a\b\f\n\r\t\v\\", "\a\b\f\n\r\t\v\\"),
        ("[[ test ]]", r"\[\[ test \]\]"),
        ("[ [ test ] ]", r"[ [ test ] ]"),
    ],
)
def test_escaped_str_multiline(val: Any, exp: str):
    class TestSchema(ConfigSchema):
        pattern: EscapedStr

    d = TestSchema({"pattern": val})
    assert (
        str_multiline_template.render(string=d.pattern)
        == f"""[[
{exp}
]]"""
    )


@pytest.mark.parametrize(
    "val,exp",
    [
        ("", ""),
        ("string", "string"),
        (2000, "2000"),
        ('"\a\b\f\n\r\t\v\\"', r"\"\a\b\f\n\r\t\v\\\""),
        ('""', r"\"\""),
        ("''", r"\'\'"),
        # fmt: off
        ('""', r"\"\""),
        ("''", r"\'\'"),
        # fmt: on
    ],
)
def test_escaped_str(val: Any, exp: str):
    class TestSchema(ConfigSchema):
        pattern: EscapedStr

    d = TestSchema({"pattern": val})
    assert str_template.render(string=d.pattern) == f"'{exp}'"



import logging
import os
import socket

logger = logging.getLogger(__name__)


def systemd_notify(**values: str) -> None:
    """
    Send systemd notify message to notify socket.

    Notify socket location (unix socket) should be saved in $NOTIFY_SOCKET environment variable.
    It is typically set by the processes supervisor (supervisord).
    If $NOTIFY_SOCKET is not configured, it is not possible to send a notification and the operation will fail.
    """
    socket_addr = os.getenv("NOTIFY_SOCKET")
    if socket_addr is None:
        logger.warning("Failed to get $NOTIFY_SOCKET environment variable")
        return

    if socket_addr.startswith("@"):
        socket_addr = socket_addr.replace("@", "\0", 1)

    try:
        notify_socket = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)
        notify_socket.connect(socket_addr)
    except OSError:
        logger.exception("Failed to connect to $NOTIFY_SOCKET at '%s'", socket_addr)
        return

    payload = "\n".join((f"{key}={value}" for key, value in values.items()))
    try:
        notify_socket.send(payload.encode("utf8"))
    except OSError:
        logger.exception("Failed to send systemd notification to $NOTIFY_SOCKET at '%s'", socket_addr)

    notify_socket.close()



import inspect
import json
from typing import Any, Dict, Type, cast

from knot_resolver.datamodel import KresConfig
from knot_resolver.datamodel.lua_schema import LuaSchema
from knot_resolver.utils.modeling import BaseSchema
from knot_resolver.utils.modeling.types import (
    get_annotations,
    get_generic_type_argument,
    get_generic_type_arguments,
    get_optional_inner_type,
    is_dict,
    is_list,
    is_optional,
    is_union,
)


def test_config_check_str_type():
    # check that there is no 'str' type in datamodel schema (except for LuaSchema
    def _check_str_type(cls: Type[Any], object_path: str = ""):
        if cls == str:
            raise TypeError(f"{object_path}: 'str' type not allowed")
        elif is_optional(cls):
            inner: Type[Any] = get_optional_inner_type(cls)
            _check_str_type(inner, object_path)
        elif is_union(cls):
            variants = get_generic_type_arguments(cls)
            for v in variants:
                _check_str_type(v, object_path)
        elif is_dict(cls):
            key_type, val_type = get_generic_type_arguments(cls)
            _check_str_type(key_type, object_path)
            _check_str_type(val_type, object_path)
        elif is_list(cls):
            inner_type = get_generic_type_argument(cls)
            _check_str_type(inner_type, object_path)

        elif inspect.isclass(cls) and issubclass(cls, BaseSchema):
            annot = get_annotations(cls)
            for name, python_type in annot.items():
                # ignore lua section
                if python_type != LuaSchema:
                    _check_str_type(python_type, f"{object_path}/{name}")

    _check_str_type(KresConfig)


def test_config_defaults():
    config = KresConfig()

    # DNS64 default
    assert config.dns64.enable == False


def test_dnssec_false():
    config = KresConfig({"dnssec": {"enable": False}})

    assert config.dnssec.enable == False


def test_dnssec_default_true():
    config = KresConfig()

    # DNSSEC defaults
    assert config.dnssec.enable == True
    assert config.dnssec.sentinel == True
    assert config.dnssec.signal_query == True
    assert config.dnssec.trust_anchors == None
    assert config.dnssec.trust_anchors_files == None
    assert config.dnssec.negative_trust_anchors == None


def test_dns64_prefix_default():
    config = KresConfig({"dns64": {"enable": True}})

    assert config.dns64.enable == True
    assert str(config.dns64.prefix) == "64:ff9b::/96"


def test_config_json_schema():
    dct = KresConfig.json_schema()

    def recser(obj: Any, path: str = "") -> None:
        if not isinstance(obj, dict):
            return
        else:
            obj = cast(Dict[Any, Any], obj)
            for key in obj:
                recser(obj[key], path=f"{path}/{key}")
            try:
                _ = json.dumps(obj)
            except BaseException as e:
                raise Exception(f"failed to serialize '{path}': {e}") from e

    recser(dct)



import copy
from typing import Any, Dict

import pytest
from pytest import raises

from knot_resolver.utils.modeling.exceptions import DataValidationError
from knot_resolver.utils.modeling.parsing import data_combine

# default data
data_default = {"key1": {"inner11": False}}


@pytest.mark.parametrize(
    "val,res",
    [
        ({"key2": "value"}, {"key1": {"inner11": False}, "key2": "value"}),
        ({"key2": {"inner21": True}}, {"key1": {"inner11": False}, "key2": {"inner21": True}}),
        ({"key1": {"inner12": 5}}, {"key1": {"inner11": False, "inner12": 5}}),
    ],
)
def test_data_combine_valid(val: Dict[Any, Any], res: Dict[Any, Any]) -> None:
    data = copy.deepcopy(data_default)
    assert data_combine(data, val) == res


@pytest.mark.parametrize("val", [{"key1": "value"}, {"key1": {"inner11": False}}])
def test_data_combine_invalid(val: Dict[Any, Any]) -> None:
    data = copy.deepcopy(data_default)
    with raises(DataValidationError):
        data_combine(data, val)



from pytest import raises

from knot_resolver.utils.modeling.query import query


def test_example_from_spec():
    # source of the example: https://jsonpatch.com/
    original = {"baz": "qux", "foo": "bar"}
    patch = [
        {"op": "replace", "path": "/baz", "value": "boo"},
        {"op": "add", "path": "/hello", "value": ["world"]},
        {"op": "remove", "path": "/foo"},
    ]
    expected = {"baz": "boo", "hello": ["world"]}

    result, _ = query(original, "patch", "", patch)

    assert result == expected



#!/usr/bin/env python3

import importlib
import importlib.util
import sys
from types import ModuleType

import pkg_resources

# replace imports with mocks
dummy = ModuleType("dummy")
dummy.__dict__["setup"] = lambda *args, **kwargs: None
dummy.__dict__["build"] = lambda *args, **kwargs: None
sys.modules["setuptools"] = dummy
sys.modules["build_c_extensions"] = dummy

# load install_requires array from setup.py
spec = importlib.util.spec_from_file_location("setup", sys.argv[1] if len(sys.argv) == 2 else "setup.py")
mod = importlib.util.module_from_spec(spec)
spec.loader.exec_module(mod)
install_requires = mod.install_requires

# strip version codes
deps = set((x[: x.index(">")].lower() if ">" in x else x.lower() for x in install_requires))

# find out which packages are missing
installed = {pkg.key for pkg in pkg_resources.working_set}
missing = deps - installed

# fail if there are some missing
if len(missing) > 0:
    print(f"Some required packages are missing: {missing}", file=sys.stderr)
    exit(1)



import pytest

from knot_resolver.datamodel.config_schema import KresConfig
from knot_resolver.manager.config_store import ConfigStore, only_on_real_changes_update


@pytest.mark.asyncio  # type: ignore
async def test_only_once():
    count = 0

    @only_on_real_changes_update(lambda config: config.logging.level)
    async def change_callback(config: KresConfig, force: bool = False) -> None:
        nonlocal count
        count += 1

    config = KresConfig()
    store = ConfigStore(config)

    await store.register_on_change_callback(change_callback)
    assert count == 1

    config = KresConfig()
    config.logging.level = "crit"
    await store.update(config)
    assert count == 2

    config = KresConfig()
    config.lua.script_only = True
    config.lua.script = "meaningless value"
    await store.update(config)
    assert count == 2



from __future__ import annotations

from functools import wraps
from typing import TYPE_CHECKING, Callable, TypeVar

if TYPE_CHECKING:
    from typing_extensions import ParamSpec

    P = ParamSpec("P")
    R = TypeVar("R")
    T = TypeVar("T")


def ignore_exceptions_optional(
    exceptions: type[BaseException] | tuple[type[BaseException]], default: T | None
) -> Callable[[Callable[P, R]], Callable[P, R | T | None]]:
    """
    Prevent exception(s) from being raised and return the configured default value instead..

    Args:
        exceptions: Exception(s) to catch.
        default: The default value to return.

    Returns:
        The value of the decorated function or the default value if an exception is caught.

    """

    def decorator(func: Callable[P, R]) -> Callable[P, T | (R | None)]:
        @wraps(func)
        def wrapper(*args: P.args, **kwargs: P.kwargs) -> T | (R | None):
            try:
                return func(*args, **kwargs)
            except exceptions:
                return default

        return wrapper

    return decorator



from knot_resolver.utils.modeling.renaming import renamed


def test_all():
    ref = {
        "awesome-customers": [{"name": "John", "home-address": "London"}, {"name": "Bob", "home-address": "Prague"}],
        "storage": {"bobby-pin": 5, "can-opener": 0, "laptop": 1},
    }

    rnm = renamed(ref)
    assert rnm["awesome_customers"][0]["home_address"] == "London"
    assert rnm["awesome_customers"][1:][0]["home_address"] == "Prague"
    assert set(rnm["storage"].items()) == set((("can_opener", 0), ("bobby_pin", 5), ("laptop", 1)))
    assert set(rnm["storage"].keys()) == set(("bobby_pin", "can_opener", "laptop"))


def test_nested_init():
    val = renamed(renamed(({"ke-y": "val-ue"})))
    assert val["ke_y"] == "val-ue"


def test_original():
    obj = renamed(({"ke-y": "val-ue"})).original()
    assert obj["ke-y"] == "val-ue"



from pyparsing import empty

from knot_resolver.utils.etag import structural_etag


def test_etag():
    empty1 = {}
    empty2 = {}

    assert structural_etag(empty1) == structural_etag(empty2)

    something1 = {"something": 1}
    something2 = {"something": 2}
    assert structural_etag(empty1) != structural_etag(something1)
    assert structural_etag(something1) != structural_etag(something2)



from pytest import raises

from knot_resolver.utils.modeling.json_pointer import json_ptr_resolve

# example adopted from https://www.sitepoint.com/json-server-example/
TEST = {
    "clients": [
        {
            "id": "59761c23b30d971669fb42ff",
            "isActive": True,
            "age": 36,
            "name": "Dunlap Hubbard",
            "gender": "male",
            "company": "CEDWARD",
            "email": "dunlaphubbard@cedward.com",
            "phone": "+1 (890) 543-2508",
            "address": "169 Rutledge Street, Konterra, Northern Mariana Islands, 8551",
        },
        {
            "id": "59761c233d8d0f92a6b0570d",
            "isActive": True,
            "age": 24,
            "name": "Kirsten Sellers",
            "gender": "female",
            "company": "EMERGENT",
            "email": "kirstensellers@emergent.com",
            "phone": "+1 (831) 564-2190",
            "address": "886 Gallatin Place, Fannett, Arkansas, 4656",
        },
        {
            "id": "59761c23fcb6254b1a06dad5",
            "isActive": True,
            "age": 30,
            "name": "Acosta Robbins",
            "gender": "male",
            "company": "ORGANICA",
            "email": "acostarobbins@organica.com",
            "phone": "+1 (882) 441-3367",
            "address": "697 Linden Boulevard, Sattley, Idaho, 1035",
        },
    ]
}


def test_json_ptr():
    parent, res, token = json_ptr_resolve(TEST, "")
    assert parent is None
    assert res is TEST

    parent, res, token = json_ptr_resolve(TEST, "/")
    assert parent is TEST
    assert res is None
    assert token == ""

    parent, res, token = json_ptr_resolve(TEST, "/clients/2/gender")
    assert parent is TEST["clients"][2]
    assert res == "male"
    assert token == "gender"

    with raises(ValueError):
        _ = json_ptr_resolve(TEST, "//")

    with raises(SyntaxError):
        _ = json_ptr_resolve(TEST, "invalid/ptr")

    with raises(ValueError):
        _ = json_ptr_resolve(TEST, "/clients/2/gender/invalid")

    parent, res, token = json_ptr_resolve(TEST, "/~01")
    assert parent is TEST
    assert res is None
    assert token == "~1"



from pytest import raises

from knot_resolver.datamodel.lua_schema import LuaSchema
from knot_resolver.utils.modeling.exceptions import DataValidationError


def test_invalid():
    with raises(DataValidationError):
        LuaSchema({"script": "-- lua script", "script-file": "path/to/file"})



import toml

from knot_resolver import __version__


def test_version():
    with open("pyproject.toml", "r") as f:
        pyproject = toml.load(f)

    version = pyproject["tool"]["poetry"]["version"]
    assert __version__ == version



from knot_resolver.datamodel.forward_schema import ForwardSchema
from knot_resolver.datamodel.templates import template_from_str
from knot_resolver.datamodel.types import IPAddressOptionalPort


def test_policy_rule_forward_add():
    tmpl_str = """{% from 'macros/forward_macros.lua.j2' import policy_rule_forward_add %}
{{ policy_rule_forward_add(rule.subtree[0],rule.options,rule.servers) }}"""

    rule = ForwardSchema(
        {
            "subtree": ".",
            "servers": [{"address": ["2001:148f:fffe::1", "185.43.135.1"], "hostname": "odvr.nic.cz"}],
            "options": {
                "authoritative": False,
                "dnssec": True,
            },
        }
    )
    result = "policy.rule_forward_add('.',{dnssec=true,auth=false},{{'2001:148f:fffe::1',tls=false,hostname='odvr.nic.cz',},{'185.43.135.1',tls=false,hostname='odvr.nic.cz',},})"

    tmpl = template_from_str(tmpl_str)
    assert tmpl.render(rule=rule) == result

    rule.servers = [IPAddressOptionalPort("2001:148f:fffe::1"), IPAddressOptionalPort("185.43.135.1")]
    result = "policy.rule_forward_add('.',{dnssec=true,auth=false},{{'2001:148f:fffe::1'},{'185.43.135.1'},})"
    assert tmpl.render(rule=rule) == result



from knot_resolver.datamodel.network_schema import ListenSchema
from knot_resolver.datamodel.templates import template_from_str


def test_network_listen():
    tmpl_str = """{% from 'macros/network_macros.lua.j2' import network_listen %}
{{ network_listen(listen) }}"""
    tmpl = template_from_str(tmpl_str)

    soc = ListenSchema({"unix-socket": "/tmp/kresd-socket", "kind": "dot"})
    assert tmpl.render(listen=soc) == "net.listen('/tmp/kresd-socket',nil,{kind='tls',freebind=false})\n"
    soc_list = ListenSchema({"unix-socket": [soc.unix_socket.to_std()[0], "/tmp/kresd-socket2"], "kind": "dot"})
    assert (
        tmpl.render(listen=soc_list)
        == "net.listen('/tmp/kresd-socket',nil,{kind='tls',freebind=false})\n"
        + "net.listen('/tmp/kresd-socket2',nil,{kind='tls',freebind=false})\n"
    )

    ip = ListenSchema({"interface": "::1@55", "freebind": True})
    assert tmpl.render(listen=ip) == "net.listen('::1',55,{kind='dns',freebind=true})\n"
    ip_list = ListenSchema({"interface": [ip.interface.to_std()[0], "127.0.0.1@5335"]})
    assert (
        tmpl.render(listen=ip_list)
        == "net.listen('::1',55,{kind='dns',freebind=false})\n"
        + "net.listen('127.0.0.1',5335,{kind='dns',freebind=false})\n"
    )

    intrfc = ListenSchema({"interface": "eth0", "kind": "doh2"})
    assert tmpl.render(listen=intrfc) == "net.listen(net['eth0'],443,{kind='doh2',freebind=false})\n"
    intrfc_list = ListenSchema({"interface": [intrfc.interface.to_std()[0], "lo"], "port": 5555, "kind": "doh2"})
    assert (
        tmpl.render(listen=intrfc_list)
        == "net.listen(net['eth0'],5555,{kind='doh2',freebind=false})\n"
        + "net.listen(net['lo'],5555,{kind='doh2',freebind=false})\n"
    )



from typing import Any, Dict, List, Literal, Tuple, Union

import pytest

from knot_resolver.utils.modeling import BaseSchema
from knot_resolver.utils.modeling.types import is_list, is_literal

types = [
    bool,
    int,
    str,
    Dict[Any, Any],
    Tuple[Any, Any],
    Union[str, int],
    BaseSchema,
]
literal_types = [Literal[5], Literal["test"], Literal[False]]


@pytest.mark.parametrize("val", types)
def test_is_list_true(val: Any):
    assert is_list(List[val])


@pytest.mark.parametrize("val", types)
def test_is_list_false(val: Any):
    assert not is_list(val)


@pytest.mark.parametrize("val", literal_types)
def test_is_literal_true(val: Any):
    assert is_literal(Literal[val])


@pytest.mark.parametrize("val", types)
def test_is_literal_false(val: Any):
    assert not is_literal(val)



import ipaddress
import random
import string
from typing import Any

import pytest
from pytest import raises

from knot_resolver.datamodel.types import (
    Dir,
    DomainName,
    EscapedStr,
    InterfaceName,
    InterfaceOptionalPort,
    InterfacePort,
    IPAddressEM,
    IPAddressOptionalPort,
    IPAddressPort,
    IPNetwork,
    IPv4Address,
    IPv6Address,
    IPv6Network96,
    PinSha256,
    PortNumber,
    SizeUnit,
    TimeUnit,
)
from knot_resolver.utils.modeling import BaseSchema


def _rand_domain(label_chars: int, levels: int = 1) -> str:
    return "".join(
        ["".join(random.choices(string.ascii_letters + string.digits, k=label_chars)) + "." for i in range(levels)]
    )


@pytest.mark.parametrize("val", [1, 65_535, 5335, 5000])
def test_port_number_valid(val: int):
    assert int(PortNumber(val)) == val


@pytest.mark.parametrize("val", [0, 65_636, -1, "53"])
def test_port_number_invalid(val: Any):
    with raises(ValueError):
        PortNumber(val)


@pytest.mark.parametrize("val", ["5368709120B", "5242880K", "5120M", "5G"])
def test_size_unit_valid(val: str):
    o = SizeUnit(val)
    assert int(o) == 5368709120
    assert str(o) == val
    assert o.bytes() == 5368709120


@pytest.mark.parametrize("val", ["-5B", 5, -5242880, "45745mB"])
def test_size_unit_invalid(val: Any):
    with raises(ValueError):
        SizeUnit(val)


@pytest.mark.parametrize("val", ["1d", "24h", "1440m", "86400s", "86400000ms"])
def test_time_unit_valid(val: str):
    o = TimeUnit(val)
    assert int(o) == 86400000000
    assert str(o) == val
    assert o.seconds() == 86400
    assert o.millis() == 86400000
    assert o.micros() == 86400000000


@pytest.mark.parametrize("val", ["-1", "-24h", "1440mm", 6575, -1440])
def test_time_unit_invalid(val: Any):
    with raises(ValueError):
        TimeUnit("-1")


def test_parsing_units():
    class TestSchema(BaseSchema):
        size: SizeUnit
        time: TimeUnit

    o = TestSchema({"size": "3K", "time": "10m"})
    assert int(o.size) == int(SizeUnit("3072B"))
    assert int(o.time) == int(TimeUnit("600s"))
    assert o.size.bytes() == 3072
    assert o.time.seconds() == 10 * 60


def test_checked_path():
    class TestSchema(BaseSchema):
        p: Dir

    assert str(TestSchema({"p": "/tmp"}).p) == "/tmp"


@pytest.mark.parametrize(
    "val",
    [
        "d6qzRu9zOECb90Uez27xWltNsj0e1Md7GkYYkVoZWmM=",
        "E9CZ9INDbd+2eRQozYqqbQ2yXLVKB9+xcprMF+44U1g=",
    ],
)
def test_pin_sha256_valid(val: str):
    o = PinSha256(val)
    assert str(o) == val


@pytest.mark.parametrize(
    "val",
    [
        "d6qzRu9zOECb90Uez27xWltNsj0e1Md7GkYYkVoZWmM==",
        "E9CZ9INDbd+2eRQozYqqbQ2yXLVKB9+xcprMF+44U1g",
        "!E9CZ9INDbd+2eRQozYqqbQ2yXLVKB9+xcprMF+44U1g=",
        "d6qzRu9zOE",
    ],
)
def test_pin_sha256_invalid(val: str):
    with raises(ValueError):
        PinSha256(val)


@pytest.mark.parametrize(
    "val,exp",
    [
        ("", r""),
        (2000, "2000"),
        ("string", r"string"),
        ("\t\n\v", r"\t\n\v"),
        ("\a\b\f\n\r\t\v\\", r"\a\b\f\n\r\t\v\\"),
        # fmt: off
        ("''", r"\'\'"),
        ('""', r"\"\""),
        ("''", r"\'\'"),
        ('""', r"\"\""),
        ('\\"\\"', r"\\\"\\\""),
        ("\\'\\'", r"\\\'\\\'"),
        # fmt: on
    ],
)
def test_escaped_str_valid(val: Any, exp: str):
    assert str(EscapedStr(val)) == exp


@pytest.mark.parametrize("val", [1.1, False])
def test_escaped_str_invalid(val: Any):
    with raises(ValueError):
        EscapedStr(val)


@pytest.mark.parametrize(
    "val",
    [
        ".",
        "example.com",
        "_8443._https.example.com.",
        "this.is.example.com.",
        "test.example.com",
        "test-example.com",
        "bücher.com.",
        "příklad.cz",
        _rand_domain(63),
        _rand_domain(1, 127),
    ],
)
def test_domain_name_valid(val: str):
    o = DomainName(val)
    assert str(o) == val
    assert o == DomainName(val)
    assert o.punycode() == val.encode("idna").decode("utf-8") if val != "." else "."


@pytest.mark.parametrize(
    "val",
    [
        "test.example..com.",
        "-example.com",
        "-test.example.net",
        "test-.example.net",
        "test.-example.net",
        ".example.net",
        _rand_domain(64),
        _rand_domain(1, 128),
    ],
)
def test_domain_name_invalid(val: str):
    with raises(ValueError):
        DomainName(val)


@pytest.mark.parametrize("val", ["lo", "eth0", "wlo1", "web_ifgrp", "e8-2"])
def test_interface_name_valid(val: str):
    assert str(InterfaceName(val)) == val


@pytest.mark.parametrize("val", ["_lo", "-wlo1", "lo_", "wlo1-", "e8--2", "web__ifgrp"])
def test_interface_name_invalid(val: Any):
    with raises(ValueError):
        InterfaceName(val)


@pytest.mark.parametrize("val", ["lo@5335", "2001:db8::1000@5001"])
def test_interface_port_valid(val: str):
    o = InterfacePort(val)
    assert str(o) == val
    assert o == InterfacePort(val)
    assert str(o.if_name if o.if_name else o.addr) == val.split("@", 1)[0]
    assert o.port == PortNumber(int(val.split("@", 1)[1]))


@pytest.mark.parametrize("val", ["lo", "2001:db8::1000", "53"])
def test_interface_port_invalid(val: Any):
    with raises(ValueError):
        InterfacePort(val)


@pytest.mark.parametrize("val", ["lo", "123.4.5.6", "lo@5335", "2001:db8::1000@5001"])
def test_interface_optional_port_valid(val: str):
    o = InterfaceOptionalPort(val)
    assert str(o) == val
    assert o == InterfaceOptionalPort(val)
    assert str(o.if_name if o.if_name else o.addr) == (val.split("@", 1)[0] if "@" in val else val)
    assert o.port == (PortNumber(int(val.split("@", 1)[1])) if "@" in val else None)


@pytest.mark.parametrize("val", ["lo@", "@53"])
def test_interface_optional_port_invalid(val: Any):
    with raises(ValueError):
        InterfaceOptionalPort(val)


@pytest.mark.parametrize("val", ["123.4.5.6@5335", "2001:db8::1000@53"])
def test_ip_address_port_valid(val: str):
    o = IPAddressPort(val)
    assert str(o) == val
    assert o == IPAddressPort(val)
    assert str(o.addr) == val.split("@", 1)[0]
    assert o.port == PortNumber(int(val.split("@", 1)[1]))


@pytest.mark.parametrize(
    "val", ["123.4.5.6", "2001:db8::1000", "123.4.5.6.7@5000", "2001:db8::10000@5001", "123.4.5.6@"]
)
def test_ip_address_port_invalid(val: Any):
    with raises(ValueError):
        IPAddressPort(val)


@pytest.mark.parametrize("val", ["123.4.5.6", "123.4.5.6@5335", "2001:db8::1000", "2001:db8::1000@53"])
def test_ip_address_optional_port_valid(val: str):
    o = IPAddressOptionalPort(val)
    assert str(o) == val
    assert o == IPAddressOptionalPort(val)
    assert str(o.addr) == (val.split("@", 1)[0] if "@" in val else val)
    assert o.port == (PortNumber(int(val.split("@", 1)[1])) if "@" in val else None)


@pytest.mark.parametrize("val", ["123.4.5.6.7", "2001:db8::10000", "123.4.5.6@", "@55"])
def test_ip_address_optional_port_invalid(val: Any):
    with raises(ValueError):
        IPAddressOptionalPort(val)


@pytest.mark.parametrize("val", ["123.4.5.6", "192.168.0.1"])
def test_ipv4_address_valid(val: str):
    o = IPv4Address(val)
    assert str(o) == val
    assert o == IPv4Address(val)


@pytest.mark.parametrize("val", ["123456", "2001:db8::1000"])
def test_ipv4_address_invalid(val: Any):
    with raises(ValueError):
        IPv4Address(val)


@pytest.mark.parametrize("val", ["2001:db8::1000", "2001:db8:85a3::8a2e:370:7334"])
def test_ipv6_address_valid(val: str):
    o = IPv6Address(val)
    assert str(o) == val
    assert o == IPv6Address(val)


@pytest.mark.parametrize("val", ["123.4.5.6", "2001::db8::1000"])
def test_ipv6_address_invalid(val: Any):
    with raises(ValueError):
        IPv6Address(val)


@pytest.mark.parametrize("val", ["10.11.12.0/24", "64:ff9b::/96"])
def test_ip_network_valid(val: str):
    o = IPNetwork(val)
    assert str(o) == val
    assert o.to_std().prefixlen == int(val.split("/", 1)[1])
    assert o.to_std() == ipaddress.ip_network(val)


@pytest.mark.parametrize("val", ["10.11.12.13/8", "10.11.12.5/128"])
def test_ip_network_invalid(val: str):
    with raises(ValueError):
        IPNetwork(val)


@pytest.mark.parametrize("val", ["fe80::/96", "64:ff9b::/96"])
def test_ipv6_96_network_valid(val: str):
    assert str(IPv6Network96(val)) == val


@pytest.mark.parametrize("val", ["fe80::/95", "10.11.12.3/96", "64:ff9b::1/96"])
def test_ipv6_96_network_invalid(val: Any):
    with raises(ValueError):
        IPv6Network96(val)


@pytest.mark.parametrize("val", ["10.10.10.5!", "::1!"])
def test_ip_address_em_valid(val: str):
    assert str(IPAddressEM(val)) == val


@pytest.mark.parametrize("val", ["10.10.10.5", "::1", "10.10.10.5!!", "::1!!"])
def test_ip_address_em_invalid(val: Any):
    with raises(ValueError):
        IPAddressEM(val)



from knot_resolver.datamodel.forward_schema import ForwardServerSchema
from knot_resolver.datamodel.templates import template_from_str


def test_boolean():
    tmpl_str = """{% from 'macros/common_macros.lua.j2' import boolean %}
{{ boolean(x) }}"""

    tmpl = template_from_str(tmpl_str)
    assert tmpl.render(x=True) == "true"
    assert tmpl.render(x=False) == "false"


def test_boolean_neg():
    tmpl_str = """{% from 'macros/common_macros.lua.j2' import boolean %}
{{ boolean(x,true) }}"""

    tmpl = template_from_str(tmpl_str)
    assert tmpl.render(x=True) == "false"
    assert tmpl.render(x=False) == "true"


def test_string_table():
    s = "any string"
    t = [s, "other string"]
    tmpl_str = """{% from 'macros/common_macros.lua.j2' import string_table %}
{{ string_table(x) }}"""

    tmpl = template_from_str(tmpl_str)
    assert tmpl.render(x=s) == f"'{s}'"
    assert tmpl.render(x=t) == f"{{'{s}','{t[1]}',}}"


def test_str2ip_table():
    s = "2001:DB8::d0c"
    t = [s, "192.0.2.1"]
    tmpl_str = """{% from 'macros/common_macros.lua.j2' import str2ip_table %}
{{ str2ip_table(x) }}"""

    tmpl = template_from_str(tmpl_str)
    assert tmpl.render(x=s) == f"kres.str2ip('{s}')"
    assert tmpl.render(x=t) == f"{{kres.str2ip('{s}'),kres.str2ip('{t[1]}'),}}"


def test_qtype_table():
    s = "AAAA"
    t = [s, "TXT"]
    tmpl_str = """{% from 'macros/common_macros.lua.j2' import qtype_table %}
{{ qtype_table(x) }}"""

    tmpl = template_from_str(tmpl_str)
    assert tmpl.render(x=s) == f"kres.type.{s}"
    assert tmpl.render(x=t) == f"{{kres.type.{s},kres.type.{t[1]},}}"


def test_servers_table():
    s = "2001:DB8::d0c"
    t = [s, "192.0.2.1"]
    tmpl_str = """{% from 'macros/common_macros.lua.j2' import servers_table %}
{{ servers_table(x) }}"""

    tmpl = template_from_str(tmpl_str)
    assert tmpl.render(x=s) == f"'{s}'"
    assert tmpl.render(x=t) == f"{{'{s}','{t[1]}',}}"
    assert tmpl.render(x=[{"address": s}, {"address": t[1]}]) == f"{{'{s}','{t[1]}',}}"


def test_tls_servers_table():
    d = ForwardServerSchema(
        # the ca-file is a dummy, because it's existence is checked
        {"address": ["2001:DB8::d0c"], "hostname": "res.example.com", "ca-file": "/etc/passwd"}
    )
    t = [
        d,
        ForwardServerSchema(
            {
                "address": "192.0.2.1",
                "pin-sha256": "E9CZ9INDbd+2eRQozYqqbQ2yXLVKB9+xcprMF+44U1g=",
            }
        ),
    ]
    tmpl_str = """{% from 'macros/common_macros.lua.j2' import tls_servers_table %}
{{ tls_servers_table(x) }}"""

    tmpl = template_from_str(tmpl_str)
    assert tmpl.render(x=[d.address, t[1].address]) == f"{{'{d.address}','{t[1].address}',}}"
    assert (
        tmpl.render(x=t)
        == f"{{{{'{d.address}',hostname='{d.hostname}',ca_file='{d.ca_file}',}},{{'{t[1].address}',pin_sha256={{'{t[1].pin_sha256}',}}}},}}"
    )



from typing import Any, Dict

from setuptools import Extension


def build(setup_kwargs: Dict[Any, Any]) -> None:
    setup_kwargs.update(
        {
            "ext_modules": [
                Extension(
                    name="knot_resolver.controller.supervisord.plugin.notify",
                    sources=["python/knot_resolver/controller/supervisord/plugin/notifymodule.c"],
                ),
            ]
        }
    )



from typing import Any

import pytest

from knot_resolver.datamodel.cache_schema import CacheClearRPCSchema
from knot_resolver.datamodel.templates import template_from_str


@pytest.mark.parametrize(
    "val,res",
    [
        ({}, "cache.clear(nil,false,nil,100)"),
        ({"chunk-size": 200}, "cache.clear(nil,false,nil,200)"),
        ({"name": "example.com.", "exact-name": True}, "cache.clear('example.com.',true,nil,nil)"),
        (
            {"name": "example.com.", "exact-name": True, "rr-type": "AAAA"},
            "cache.clear('example.com.',true,kres.type.AAAA,nil)",
        ),
    ],
)
def test_cache_clear(val: Any, res: Any):
    tmpl_str = "{% from 'macros/cache_macros.lua.j2' import cache_clear %}{{ cache_clear(x) }}"

    tmpl = template_from_str(tmpl_str)
    assert tmpl.render(x=CacheClearRPCSchema(val)) == res



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Original source:
# https://github.com/sdss/flicamera/blob/main/create_setup.py
# We modified the script so that it outputs the setup.py to stdout and that no
# version upper bounds are outputted in the depencency list.
#
# @Author: José Sánchez-Gallego (gallegoj@uw.edu)
# @Date: 2019-12-18
# @Filename: create_setup.py
# @License: BSD 3-clause (http://www.opensource.org/licenses/BSD-3-Clause)

# This is a temporary solution for the fact that pip install . fails with
# poetry when there is no setup.py and an extension needs to be compiled.
# See https://github.com/python-poetry/poetry/issues/1516. Running this
# script creates a setup.py filled out with information generated by
# poetry when parsing the pyproject.toml.

import os
import re
import sys

from packaging.version import Version

# If there is a global installation of poetry, prefer that.
lib = os.path.expanduser("~/.poetry/lib")
vendors = os.path.join(lib, "poetry", "_vendor")
current_vendors = os.path.join(vendors, "py{}".format(".".join(str(v) for v in sys.version_info[:2])))

sys.path.insert(0, lib)
sys.path.insert(0, current_vendors)

try:
    try:
        from poetry.core.factory import Factory
        from poetry.core.masonry.builders.sdist import SdistBuilder
    except (ImportError, ModuleNotFoundError):
        from poetry.factory import Factory
        from poetry.masonry.builders.sdist import SdistBuilder
    from poetry.__version__ import __version__
except (ImportError, ModuleNotFoundError) as ee:
    raise ImportError(f"install poetry by doing pip install poetry to use this script: {ee}")


# Generate a Poetry object that knows about the metadata in pyproject.toml
factory = Factory()
poetry = factory.create_poetry(os.path.dirname(__file__))

# Use the SdistBuilder to genrate a blob for setup.py
if Version(__version__) >= Version("1.1.0b1"):
    sdist_builder = SdistBuilder(poetry, None)
else:
    sdist_builder = SdistBuilder(poetry, None, None)

setuppy_blob: bytes = sdist_builder.build_setup()


# patch the result so that it does not contain upper bounds in dependencies
# (but it should contain them in python version)
setuppy = setuppy_blob.decode("utf8")
setuppy, _ = re.subn(r"(\'[^\']+>=[^<>=,\']*),<[^<>=,\']*\'", "\\1'", setuppy)

# output the setup.py script to stdout
print(setuppy)
print("\n# This setup.py was autogenerated using Poetry for backward compatibility with setuptools.")



from knot_resolver.utils.functional import all_matches, contains_element_matching, foldl


def test_foldl():
    lst = list(range(10))

    assert foldl(lambda x, y: x + y, 0, lst) == sum(range(10))
    assert foldl(lambda x, y: x + y, 55, lst) == sum(range(10)) + 55


def test_containsElementMatching():
    lst = list(range(10))

    assert contains_element_matching(lambda e: e == 5, lst)
    assert not contains_element_matching(lambda e: e == 11, lst)


def test_matches_all():
    lst = list(range(10))

    assert all_matches(lambda x: x >= 0, lst)
    assert not all_matches(lambda x: x % 2 == 0, lst)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'python'}

packages = \
['knot_resolver',
 'knot_resolver.client',
 'knot_resolver.client.commands',
 'knot_resolver.controller',
 'knot_resolver.controller.supervisord',
 'knot_resolver.controller.supervisord.plugin',
 'knot_resolver.datamodel',
 'knot_resolver.datamodel.templates',
 'knot_resolver.datamodel.types',
 'knot_resolver.manager',
 'knot_resolver.manager.files',
 'knot_resolver.manager.metrics',
 'knot_resolver.utils',
 'knot_resolver.utils.compat',
 'knot_resolver.utils.modeling']

package_data = \
{'': ['*'], 'knot_resolver.datamodel.templates': ['macros/*']}

install_requires = \
['aiohttp', 'jinja2', 'pyyaml', 'supervisor', 'typing-extensions']

extras_require = \
{'prometheus': ['prometheus-client'], 'watchdog': ['watchdog']}

entry_points = \
{'console_scripts': ['knot-resolver = knot_resolver.manager.main:main',
                     'kresctl = knot_resolver.client.main:main']}

setup_kwargs = {
    'name': 'knot-resolver',
    'version': '6.3.0',
    'description': 'Knot Resolver Manager - a Python program that automatically manages the other components of the resolver',
    'long_description': "# Knot Resolver\n\n[![Build Status](https://gitlab.nic.cz/knot/knot-resolver/badges/nightly/pipeline.svg?x)](https://gitlab.nic.cz/knot/knot-resolver/commits/nightly)\n[![Coverage Status](https://gitlab.nic.cz/knot/knot-resolver/badges/nightly/coverage.svg?x)](https://www.knot-resolver.cz/documentation/latest)\n[![Packaging status](https://repology.org/badge/tiny-repos/knot-resolver.svg)](https://repology.org/project/knot-resolver/versions)\n\nKnot Resolver is a full caching DNS resolver implementation. The core architecture is tiny and efficient, written in C and [LuaJIT][luajit], providing a foundation and a state-machine-like API for extension modules. There are three built-in modules - *iterator*, *validator* and *cache* - which provide the main functionality of the resolver. A few other modules are automatically loaded by default to extend the resolver's functionality.\n\nSince Knot Resolver version 6, it also includes a so-called [manager][manager]. It is a new component written in [Python][python] that hides the complexity of older versions and makes it more user friendly. For example, new features include declarative configuration in YAML format and HTTP API for dynamic changes in the resolver and more.\n\nKnot Resolver uses a [different scaling strategy][scaling] than the rest of the DNS resolvers - no threading, shared-nothing architecture (except MVCC cache which can be shared), which allows you to pin workers to available CPU cores and grow by self-replication. You can start and stop additional workers based on the contention without downtime, which is automated by the [manager][manager] by default.\n\nThe LuaJIT modules, support for DNS privacy and DNSSEC, and persistent cache with low memory footprint make it a great personal DNS resolver or a research tool to tap into DNS data. Strong filtering rules, and auto-configuration with etcd make it a great large-scale resolver solution. It also has strong support for DNS over TCP, in particular TCP Fast-Open, query pipelining and deduplication, and response reordering.\n\nFor more on using the resolver, see the [User Documentation][doc]. See the [Developer Documentation][doc-dev] for detailed architecture and development.\n\n## Packages\n\nThe latest stable packages for various distributions are available in our\n[upstream repository](https://pkg.labs.nic.cz/doc/?project=knot-resolver).\nFollow the installation instructions to add this repository to your system.\n\nKnot Resolver is also available from the following distributions' repositories:\n\n* [Fedora and Fedora EPEL](https://src.fedoraproject.org/rpms/knot-resolver)\n* [Debian stable](https://packages.debian.org/stable/knot-resolver),\n  [Debian testing](https://packages.debian.org/testing/knot-resolver),\n  [Debian unstable](https://packages.debian.org/sid/knot-resolver)\n* [Ubuntu](https://packages.ubuntu.com/jammy/knot-resolver)\n* [Arch Linux](https://archlinux.org/packages/extra/x86_64/knot-resolver/)\n* [Alpine Linux](https://pkgs.alpinelinux.org/packages?name=knot-resolver)\n\n### Packaging\n\nThe project uses [`apkg`](https://gitlab.nic.cz/packaging/apkg) for packaging.\nSee [`distro/README.md`](distro/README.md) for packaging specific instructions.\n\n## Building from sources\n\nKnot Resolver mainly depends on [KnotDNS][knot-dns] libraries, [LuaJIT][luajit], [libuv][libuv] and [Python][python].\n\nSee the [Building project][build] documentation page for more information.\n\n## Running\n\nBy default, Knot Resolver comes with [systemd][systemd] integration and you just need to start its service. It requires no configuration changes to run a server on localhost.\n\n```\n# systemctl start knot-resolver\n```\n\nSee the documentation at [knot-resolver.cz/documentation/latest][doc] for more information.\n\n## Running the Docker image\n\nRunning the Docker image is simple and doesn't require any dependencies or system modifications, just run:\n\n```\n$ docker run -Pit cznic/knot-resolver\n```\n\nThe images are meant as an easy way to try the resolver, and they're not designed for production use.\n\n## Contacting us\n\n- [GitLab issues](https://gitlab.nic.cz/knot/knot-resolver/issues) (you may authenticate via GitHub)\n- [mailing list](https://lists.nic.cz/postorius/lists/knot-resolver-announce.lists.nic.cz/)\n- [![Join the chat at https://gitter.im/CZ-NIC/knot-resolver](https://badges.gitter.im/Join%20Chat.svg)](https://gitter.im/CZ-NIC/knot-resolver?utm_source=badge&utm_medium=badge&utm_campaign=pr-badge&utm_content=badge)\n\n[build]: https://www.knot-resolver.cz/documentation/latest/dev/build.html\n[doc]: https://www.knot-resolver.cz/documentation/latest/\n[doc-dev]: https://www.knot-resolver.cz/documentation/latest/dev\n[knot-dns]: https://www.knot-dns.cz/\n[luajit]: https://luajit.org/\n[libuv]: http://libuv.org\n[python]: https://www.python.org/\n[systemd]: https://systemd.io/\n[scaling]: https://www.knot-resolver.cz/documentation/latest/config-multiple-workers.html\n[manager]: https://www.knot-resolver.cz/documentation/latest/dev/architecture.html\n",
    'author': 'Aleš Mrázek',
    'author_email': 'ales.mrazek@nic.cz',
    'maintainer': 'Aleš Mrázek',
    'maintainer_email': 'ales.mrazek@nic.cz',
    'url': 'https://www.knot-resolver.cz',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}
from build_c_extensions import *
build(setup_kwargs)

setup(**setup_kwargs)


# This setup.py was autogenerated using Poetry for backward compatibility with setuptools.



from typing import Any

import pytest

from knot_resolver.datamodel.templates import template_from_str
from knot_resolver.datamodel.view_schema import ViewOptionsSchema, ViewSchema


def test_view_flags():
    tmpl_str = """{% from 'macros/view_macros.lua.j2' import view_flags %}
{{ view_flags(options) }}"""

    tmpl = template_from_str(tmpl_str)
    options = ViewOptionsSchema({"dns64": False, "minimize": False})
    assert tmpl.render(options=options) == '"NO_MINIMIZE","DNS64_DISABLE",'
    assert tmpl.render(options=ViewOptionsSchema()) == ""


def test_view_answer():
    tmpl_str = """{% from 'macros/view_macros.lua.j2' import view_options_flags %}
{{ view_options_flags(options) }}"""

    tmpl = template_from_str(tmpl_str)
    options = ViewOptionsSchema({"dns64": False, "minimize": False})
    assert tmpl.render(options=options) == "policy.FLAGS({'NO_MINIMIZE','DNS64_DISABLE',})"
    assert tmpl.render(options=ViewOptionsSchema()) == "policy.FLAGS({})"


@pytest.mark.parametrize(
    "val,res",
    [
        ("allow", "policy.TAGS_ASSIGN({})"),
        ("refused", "'policy.REFUSE'"),
        ("noanswer", "'policy.NO_ANSWER'"),
    ],
)
def test_view_answer(val: Any, res: Any):
    tmpl_str = """{% from 'macros/view_macros.lua.j2' import view_answer %}
{{ view_answer(view.answer) }}"""

    tmpl = template_from_str(tmpl_str)
    view = ViewSchema({"subnets": ["10.0.0.0/8"], "answer": val})
    assert tmpl.render(view=view) == res



from typing import List

from knot_resolver.datamodel.network_schema import AddressRenumberingSchema
from knot_resolver.datamodel.templates import template_from_str


def test_policy_add():
    rule = "policy.all(policy.DENY)"
    tmpl_str = """{% from 'macros/policy_macros.lua.j2' import policy_add %}
{{ policy_add(rule, postrule) }}"""

    tmpl = template_from_str(tmpl_str)
    assert tmpl.render(rule=rule, postrule=False) == f"policy.add({rule})"
    assert tmpl.render(rule=rule, postrule=True) == f"policy.add({rule},true)"


def test_policy_tags_assign():
    tags: List[str] = ["t01", "t02", "t03"]
    tmpl_str = """{% from 'macros/policy_macros.lua.j2' import policy_tags_assign %}
{{ policy_tags_assign(tags) }}"""

    tmpl = template_from_str(tmpl_str)
    assert tmpl.render(tags=tags[1]) == f"policy.TAGS_ASSIGN('{tags[1]}')"
    assert tmpl.render(tags=tags) == "policy.TAGS_ASSIGN({" + ",".join([f"'{x}'" for x in tags]) + ",})"


def test_policy_get_tagset():
    tags: List[str] = ["t01", "t02", "t03"]
    tmpl_str = """{% from 'macros/policy_macros.lua.j2' import policy_get_tagset %}
{{ policy_get_tagset(tags) }}"""

    tmpl = template_from_str(tmpl_str)
    assert tmpl.render(tags=tags[1]) == f"policy.get_tagset('{tags[1]}')"
    assert tmpl.render(tags=tags) == "policy.get_tagset({" + ",".join([f"'{x}'" for x in tags]) + ",})"


# Filters


def test_policy_all():
    action = "policy.DENY"
    tmpl_str = """{% from 'macros/policy_macros.lua.j2' import policy_all %}
{{ policy_all(action) }}"""

    tmpl = template_from_str(tmpl_str)
    assert tmpl.render(action=action) == f"policy.all({action})"


def test_policy_suffix():
    action = "policy.DROP"
    suffix = "policy.todnames({'example.com'})"
    tmpl_str = """{% from 'macros/policy_macros.lua.j2' import policy_suffix %}
{{ policy_suffix(action, suffix) }}"""

    tmpl = template_from_str(tmpl_str)
    assert tmpl.render(action=action, suffix=suffix) == f"policy.suffix({action},{suffix})"


def test_policy_suffix_common():
    action = "policy.DROP"
    suffix = "policy.todnames({'first.example.com','second.example.com'})"
    common = "policy.todnames({'example.com'})"
    tmpl_str = """{% from 'macros/policy_macros.lua.j2' import policy_suffix_common %}
{{ policy_suffix_common(action, suffix, common) }}"""

    tmpl = template_from_str(tmpl_str)
    assert tmpl.render(action=action, suffix=suffix, common=None) == f"policy.suffix_common({action},{suffix})"
    assert (
        tmpl.render(action=action, suffix=suffix, common=common) == f"policy.suffix_common({action},{suffix},{common})"
    )


def test_policy_pattern():
    action = "policy.DENY"
    pattern = "[0-9]+\2cz"
    tmpl_str = """{% from 'macros/policy_macros.lua.j2' import policy_pattern %}
{{ policy_pattern(action, pattern) }}"""

    tmpl = template_from_str(tmpl_str)
    assert tmpl.render(action=action, pattern=pattern) == f"policy.pattern({action},'{pattern}')"


def test_policy_rpz():
    action = "policy.DENY"
    path = "/etc/knot-resolver/blocklist.rpz"
    tmpl_str = """{% from 'macros/policy_macros.lua.j2' import policy_rpz %}
{{ policy_rpz(action, path, watch) }}"""

    tmpl = template_from_str(tmpl_str)
    assert tmpl.render(action=action, path=path, watch=False) == f"policy.rpz({action},'{path}',false)"
    assert tmpl.render(action=action, path=path, watch=True) == f"policy.rpz({action},'{path}',true)"


# Non-chain actions


def test_policy_deny_msg():
    msg = "this is deny message"
    tmpl_str = """{% from 'macros/policy_macros.lua.j2' import policy_deny_msg %}
{{ policy_deny_msg(msg) }}"""

    tmpl = template_from_str(tmpl_str)
    assert tmpl.render(msg=msg) == f"policy.DENY_MSG('{msg}')"


def test_policy_reroute():
    r: List[AddressRenumberingSchema] = [
        AddressRenumberingSchema({"source": "192.0.2.0/24", "destination": "127.0.0.0"}),
        AddressRenumberingSchema({"source": "10.10.10.0/24", "destination": "192.168.1.0"}),
    ]
    tmpl_str = """{% from 'macros/policy_macros.lua.j2' import policy_reroute %}
{{ policy_reroute(reroute) }}"""

    tmpl = template_from_str(tmpl_str)
    assert (
        tmpl.render(reroute=r)
        == f"policy.REROUTE({{['{r[0].source}']='{r[0].destination}'}},{{['{r[1].source}']='{r[1].destination}'}},)"
    )



import random
import sys
from typing import List, Optional

import pytest
from pytest import raises

from knot_resolver import KresBaseError
from knot_resolver.datamodel.types.base_types import FloatRangeBase, IntRangeBase, StringLengthBase


@pytest.mark.parametrize("min,max", [(0, None), (None, 0), (1, 65535), (-65535, -1)])
def test_int_range_base(min: Optional[int], max: Optional[int]):
    class Test(IntRangeBase):
        if min:
            _min = min
        if max:
            _max = max

    if min:
        assert int(Test(min)) == min
    if max:
        assert int(Test(max)) == max

    rmin = min if min else -sys.maxsize - 1
    rmax = max if max else sys.maxsize

    n = 100
    vals: List[int] = [random.randint(rmin, rmax) for _ in range(n)]
    assert [str(Test(val)) == f"{val}" for val in vals]

    invals: List[int] = []
    invals.extend([random.randint(rmax + 1, sys.maxsize) for _ in range(n % 2)] if max else [])
    invals.extend([random.randint(-sys.maxsize - 1, rmin - 1) for _ in range(n % 2)] if max else [])

    for inval in invals:
        with raises(KresBaseError):
            Test(inval)


@pytest.mark.parametrize("min,max", [(0.0, None), (None, 0.0), (1.0, 65535.0), (-65535.0, -1.0)])
def test_float_range_base(min: Optional[float], max: Optional[float]):
    class Test(FloatRangeBase):
        if min:
            _min = min
        if max:
            _max = max

    if min:
        assert float(Test(min)) == min
    if max:
        assert float(Test(max)) == max

    rmin = min if min else sys.float_info.min - 1.0
    rmax = max if max else sys.float_info.max

    n = 100
    vals: List[float] = [random.uniform(rmin, rmax) for _ in range(n)]
    assert [str(Test(val)) == f"{val}" for val in vals]

    invals: List[float] = []
    invals.extend([random.uniform(rmax + 1.0, sys.float_info.max) for _ in range(n % 2)] if max else [])
    invals.extend([random.uniform(sys.float_info.min - 1.0, rmin - 1.0) for _ in range(n % 2)] if max else [])

    for inval in invals:
        with raises(KresBaseError):
            Test(inval)


@pytest.mark.parametrize("min,max", [(10, None), (None, 10), (2, 32)])
def test_str_bytes_length_base(min: Optional[int], max: Optional[int]):
    class Test(StringLengthBase):
        if min:
            _min_bytes = min
        if max:
            _max_bytes = max

    if min:
        assert len(str(Test("x" * min)).encode("utf-8")) == min
    if max:
        assert len(str(Test("x" * max)).encode("utf-8")) == max

    n = 100
    rmin = 1 if not min else min
    rmax = 1024 if not max else max
    vals: List[str] = ["x" * random.randint(rmin, rmax) for _ in range(n)]
    assert [str(Test(val)) == f"{val}" for val in vals]

    invals: List[str] = []
    invals.extend(["x" * random.randint(rmax + 1, 2048) for _ in range(n % 2)] if max else [])
    invals.extend(["x" * random.randint(1, rmin - 1) for _ in range(n % 2)] if max else [])

    for inval in invals:
        with raises(KresBaseError):
            Test(inval)
